@@ -4,7 +4,9 @@
  * GChemPaint library
  * application.cc 
  *
- * Copyright (C) 2004-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2005
+ *
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -22,16 +24,6 @@
  * USA
  */
 
-#ifndef HAS_OPENBABEL_2
-#	include <mol.h>
-#	undef PACKAGE
-#	undef PACKAGE_BUGREPORT
-#	undef PACKAGE_NAME
-#	undef PACKAGE_STRING
-#	undef PACKAGE_TARNAME
-#	undef PACKAGE_VERSION
-#	undef VERSION
-#endif
 #include "gchempaint-config.h"
 #include "application.h"
 #include "document.h"
@@ -47,19 +39,16 @@
 #include "reactant.h"
 #include "reaction-step.h"
 #include "reaction-arrow.h"
+#include "zoomdlg.h"
 #include <bonobo/bonobo-ui-util.h>
 #include <sys/stat.h>
 #include <gconf/gconf-client.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
-#include <clocale>
-#ifdef HAS_OPENBABEL_2
-#	include <openbabel/mol.h>
-#	include <openbabel/obconversion.h>
-#else
-#	include <sstream>
-#endif
+#include <locale.h>
+#include <openbabel/mol.h>
+#include <openbabel/obconversion.h>
 
 // Objects creation static methods
 static Object* CreateMolecule()
@@ -115,9 +104,14 @@ static Object* CreateFragment()
 list<gcpApplication*> Apps;
 bool	gcpApplication::m_bInit = false;
 bool	gcpApplication::m_Have_Ghemical = false;
+bool	gcpApplication::m_Have_InChI = false;
 map<string, string> gcpApplication::m_SupportedPixbufFormats;
 
-gcpApplication::gcpApplication ()
+#warning "the following lines should be removed for stable releases"
+#undef PACKAGE
+#define PACKAGE "gchempaint-unstable" 
+
+gcpApplication::gcpApplication (): Application (PACKAGE, DATADIR)
 {
 	Apps.push_back(this);
 	m_CurZ = 6;
@@ -129,12 +123,12 @@ gcpApplication::gcpApplication ()
 	GConfClient* cli = gconf_client_get_default ();
 	if (cli) {
 		const char *value;
-		GConfEntry* entry = gconf_client_get_entry (cli, "/desktop/gnome/applications/help_viewer/exec", NULL, true, NULL);
+/*		GConfEntry* entry = gconf_client_get_entry (cli, "/desktop/gnome/applications/help_viewer/exec", NULL, true, NULL);
 		if (entry) {
 			value = gconf_value_get_string (gconf_entry_get_value (entry));
 			if (value) HelpBrowser = value;
-		}
-		entry = gconf_client_get_entry (cli, "/desktop/gnome/applications/browser/exec", NULL, true, NULL);
+		}*/
+		GConfEntry* entry = gconf_client_get_entry (cli, "/desktop/gnome/applications/browser/exec", NULL, true, NULL);
 		if (entry) {
 			value = gconf_value_get_string (gconf_entry_get_value (entry));
 			if (value) WebBrowser = value;
@@ -145,36 +139,32 @@ gcpApplication::gcpApplication ()
 			if (value) {
 				MailAgent = value;
 				int i = MailAgent.find (" %s");
-				if (i > 0)
+				if (i)
 					MailAgent.erase (i, MailAgent.size ());
 			}
 		}
 	}
 
 	// search help file
-	char const *szlang = getenv ("LANG");
-	string lang = (szlang)? szlang: "C";
-	HelpFilename = string ("file://"DATADIR"/gnome/help/") + string (PACKAGE"/") + lang + "/"PACKAGE".xml";
-	GnomeVFSURI *uri = gnome_vfs_uri_new (HelpFilename.c_str ());
-	bool exists = gnome_vfs_uri_exists (uri);
-	gnome_vfs_uri_unref (uri);
-	if (!exists) {
+/*	string lang = getenv ("LANG");
+	HelpFilename = string (DATADIR"/gnome/help/") + string (PACKAGE"/") + lang + "/"PACKAGE".xml";
+	struct stat buf;
+	int err = stat(HelpFilename.c_str (), &buf);
+	if (err) {
 		if (lang.size () > 2) {
 			lang.erase (2, lang.size());
-			HelpFilename = string ("file://"DATADIR"/gnome/help/"PACKAGE"/") + lang + "/"PACKAGE".xml";
-			uri = gnome_vfs_uri_new (HelpFilename.c_str ());
+			HelpFilename = string (DATADIR"/gnome/help/"PACKAGE"/") + lang + "/"PACKAGE".xml";
+			err = stat(HelpFilename.c_str (), &buf);
 		}
-		if (!exists) {
-			HelpFilename = "file://"DATADIR"/gnome/help/"PACKAGE"/C/"PACKAGE".xml";
-			uri = gnome_vfs_uri_new (HelpFilename.c_str ());
-			exists = gnome_vfs_uri_exists (uri);
-			gnome_vfs_uri_unref (uri);
+		if (err) {
+			HelpFilename = DATADIR"/gnome/help/"PACKAGE"/C/"PACKAGE".xml";
+			err = stat(HelpFilename.c_str (), &buf);
 		}
 	}
-	if (!exists) {
+	if (err) {
 		g_warning("Help file not found");
 		HelpFilename = "";
-	}
+	}*/
 
 	if (!m_bInit) {
 		// Check for programs
@@ -182,6 +172,18 @@ gcpApplication::gcpApplication ()
 		// check for ghemical
 		m_Have_Ghemical = (g_spawn_command_line_sync ("which ghemical", &result, &errors, NULL, NULL)
 			&& result && strlen (result));
+		if (result) {
+			g_free (result);
+			result = NULL;
+		}
+		if (errors) {
+			g_free (errors);
+			errors = NULL;
+		}
+		OBConversion Conv;
+		m_Have_InChI = Conv.FindFormat ("inchi") != NULL || 
+			(g_spawn_command_line_sync ("which main_inchi", &result, &errors, NULL, NULL)
+			&& result && strlen (result));
 		if (result)
 			g_free (result);
 		if (errors) {
@@ -309,7 +311,7 @@ void gcpApplication::AddUI (const char* filename)
 	else 
 	{
 		BonoboUINode *Child, *Next;
-		if (!strcmp(bonobo_ui_node_get_name (Parent), "Root"))
+		if (!strcmp (bonobo_ui_node_get_name (Parent), "Root"))
 			Node = bonobo_ui_node_children (Parent);
 		else
 			Node = NULL;
@@ -432,25 +434,11 @@ bool gcpApplication::FileProcess(const gchar* filename, int filetype, bool bSave
 		{
 			// destroy the old file
 			gnome_vfs_unlink (filename2);
-#ifdef HAS_OPENBABEL_2
 			pDoc->SetType (filetype);
 			if (filetype == gcp_ft_Normal)
 				SaveGcp (filename2, pDoc);
 			else
 				SaveWithBabel (filename2, ext, pDoc, filetype);
-#else
-			switch(filetype)
-			{
-				case gcp_ft_Normal:
-					pDoc->SetType(UNDEFINED);
-					SaveGcp(filename2, pDoc);
-					break;
-				case gcp_ft_CML: SaveWithBabel(filename2, ext, pDoc, CML); break;
-				case gcp_ft_SDF: SaveWithBabel(filename2, ext, pDoc, SDF); break;
-				case gcp_ft_PDB: SaveWithBabel(filename2, ext, pDoc, PDB); break;
-				case gcp_ft_ChemDraw: SaveWithBabel(filename2, ext, pDoc, CHEMDRAW); break;
-			}
-#endif
 		}
 	}
 	else //loading
@@ -518,21 +506,10 @@ bool gcpApplication::FileProcess(const gchar* filename, int filetype, bool bSave
 			}
 		}
 		else filename2 = g_strdup(filename);
-#ifdef HAS_OPENBABEL_2
 		if (filetype == gcp_ft_Normal)
 			OpenGcp (filename2, pDoc);
 		else
 			OpenWithBabel (filename2, ext, pDoc, filetype);
-#else
-		switch (filetype)
-		{
-			case gcp_ft_Normal: OpenGcp(filename2, pDoc); break;
-			case gcp_ft_CML: OpenWithBabel(filename2, ext, pDoc, CML); break;
-			case gcp_ft_SDF: OpenWithBabel(filename2, ext, pDoc, SDF); break;
-			case gcp_ft_PDB: OpenWithBabel(filename2, ext, pDoc, PDB); break;
-			case gcp_ft_ChemDraw: OpenWithBabel(filename2, ext, pDoc, CHEMDRAW); break;
-		}
-#endif
 	}
 	g_free(filename2);
 	return false;
@@ -568,17 +545,11 @@ void gcpApplication::OpenWithBabel(const gchar* filename, const gchar* ext, gcpD
 			if (!ifs) throw (int) 1;
 			old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 			setlocale(LC_NUMERIC, "C");
-#ifdef HAS_OPENBABEL_2
 			OBMol Mol;
 			OBConversion Conv;
 			OBFormat* pInFormat = Conv.FormatFromExt (filename);
 			Conv.SetInAndOutFormats (pInFormat, pInFormat);
 			Conv.Read (&Mol, &ifs);
-#else
-			OBMol Mol ((io_type) Type);
-			OBFileFormat fileFormat;
-			fileFormat.ReadMolecule (ifs, Mol);
-#endif
 			result = pDoc->ImportOB(Mol);
 			setlocale (LC_NUMERIC, old_num_locale);
 			g_free (old_num_locale);
@@ -591,17 +562,11 @@ void gcpApplication::OpenWithBabel(const gchar* filename, const gchar* ext, gcpD
 			istringstream iss (buf);
 			old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 			setlocale(LC_NUMERIC, "C");
-#ifdef HAS_OPENBABEL_2
 			OBMol Mol;
 			OBConversion Conv;
 			OBFormat* pInFormat = Conv.FormatFromExt (filename);
 			Conv.SetInAndOutFormats (pInFormat, pInFormat);
 			Conv.Read (&Mol, &iss);
-#else
-			OBMol Mol ((io_type) Type);
-			OBFileFormat fileFormat;
-			fileFormat.ReadMolecule (iss, Mol);
-#endif
 			result = pDoc->ImportOB(Mol);
 			setlocale (LC_NUMERIC, old_num_locale);
 			setlocale (LC_NUMERIC, old_num_locale);
@@ -767,7 +732,7 @@ void gcpApplication::InitTools()
 			(*i).second->Activate ((*i).first == "Select");
 }
 
-void gcpApplication::OnHelp (char* tag)
+/*void gcpApplication::OnHelp (char* tag)
 {
 	if (!HelpBrowser.size () || !HelpFilename.size ())
 		return;
@@ -779,14 +744,25 @@ void gcpApplication::OnHelp (char* tag)
 	argv[1] = (char*) path.c_str ();
 	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
 		NULL, NULL, NULL, NULL);
-}
+}*/
 
 void gcpApplication::OnBug ()
 {
 	if (!WebBrowser.size ())
 		return;
-	char *argv[3] = {NULL, const_cast<char*> (PACKAGE_BUGREPORT), NULL};
-	argv[0] = const_cast<char*> (WebBrowser.c_str());
+	char *argv[3] = {NULL, PACKAGE_BUGREPORT, NULL};
+	argv[0] = (char*) WebBrowser.c_str();
+	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
+		NULL, NULL, NULL, NULL);
+}
+
+void gcpApplication::ShowURI (string& uri)
+{
+	if (!WebBrowser.size ())
+		return;
+	char *argv[3] = {NULL, NULL, NULL};
+	argv[0] = (char*) WebBrowser.c_str();
+	argv[1] = (char*) uri.c_str ();
 	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
 		NULL, NULL, NULL, NULL);
 }
@@ -795,7 +771,7 @@ void gcpApplication::OnWeb ()
 {
 	if (!WebBrowser.size ())
 		return;
-	char *argv[3] = {NULL, const_cast<char*> ("http://www.nongnu.org/gchempaint"), NULL};
+	char *argv[3] = {NULL, "http://www.nongnu.org/gchempaint", NULL};
 	argv[0] = (char*) WebBrowser.c_str();
 	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
 		NULL, NULL, NULL, NULL);
@@ -805,7 +781,7 @@ void gcpApplication::OnMail ()
 {
 	if (!MailAgent.size ())
 		return;
-	char *argv[3] = {NULL, const_cast<char*> ("mailto:gchempaint-main@nongnu.org"), NULL};
+	char *argv[3] = {NULL, "mailto:gchempaint-main@nongnu.org", NULL};
 	argv[0] = (char*) MailAgent.c_str();
 	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
 		NULL, NULL, NULL, NULL);
@@ -893,3 +869,18 @@ void gcpApplication::SetCurDir (char const* dir)
 		g_free (CurDir);
 	CurDir = g_strdup (dir);
 }
+
+void gcpApplication::Zoom (double zoom)
+{
+	gcpView *pView = m_pActiveDoc->GetView ();
+	// authorized zooms: 20% to 800% all other values will open the zoom dialog.
+	if (zoom >= 0.2 && zoom <= 8.)
+		pView->Zoom (zoom);
+	else {
+		Dialog *pDialog = GetDialog ("Zoom");
+		if (pDialog)
+			gtk_window_present (pDialog->GetWindow ()); 
+		else
+			new gcpZoomDlg (this);
+	}
+}
