@@ -4,7 +4,9 @@
  * GChemPaint library
  * atom.cc
  *
- * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2005
+ *
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -22,17 +24,6 @@
  * USA
  */
 
-#ifndef HAS_OPENBABEL_2
-#	include <mol.h>
-#	undef PACKAGE
-#	undef PACKAGE_BUGREPORT
-#	undef PACKAGE_NAME
-#	undef PACKAGE_STRING
-#	undef PACKAGE_TARNAME
-#	undef PACKAGE_VERSION
-#	undef VERSION
-#endif
-
 #include "gchempaint-config.h"
 #include "atom.h"
 #include "electron.h"
@@ -45,22 +36,17 @@
 #include "libgcpcanvas/gcp-canvas-rect-ellipse.h"
 #include "libgcpcanvas/gcp-canvas-bpath.h"
 #include "libgcpcanvas/gcp-canvas-rich-text.h"
-#include "libgcpcanvas/gcp-canvas-text.h"
+#include "libgcpcanvas/gcp-canvas-pango.h"
 #include <math.h>
 #include <gcu/element.h>
-#ifdef HAS_OPENBABEL_2
-#	include <openbabel/mol.h>
-#endif
+#include <openbabel/mol.h>
 
 using namespace gcu;
 
-#define ATOM_EPSILON 0.1
-
 extern GtkTextTagTable* TextTagTable;
 
 gcpAtom::gcpAtom(): Atom()
 {
-	m_buf = gtk_text_buffer_new(TextTagTable);
 	m_Valence = -1; //unspecified
 	m_nlp = 0;
 	m_nH = 0;
@@ -75,6 +61,8 @@ gcpAtom::gcpAtom(): Atom()
 	m_ChargeAngle = 0.;
 	m_ChargeDist = 0.;
 	m_ChargeAutoPos = true;
+	m_Layout = m_ChargeLayout = NULL;
+	m_DrawCircle = false;
 }
 
 gcpAtom::~gcpAtom()
@@ -90,11 +78,14 @@ gcpAtom::~gcpAtom()
 		delete electron;
 		electron = GetNextChild (i);
 	}
+	if (m_Layout)
+		g_object_unref (G_OBJECT (m_Layout));
+	if (m_ChargeLayout)
+		g_object_unref (G_OBJECT (m_ChargeLayout));
 }
 
 gcpAtom::gcpAtom(int Z, double x, double y, double z): Atom(Z, x, y, z)
 {
-	m_buf = gtk_text_buffer_new(TextTagTable);
 	m_ChargeAuto = false;
 	m_HPos = GetBestSide();
 	m_nlp = 0;
@@ -108,11 +99,12 @@ gcpAtom::gcpAtom(int Z, double x, double y, double z): Atom(Z, x, y, z)
 	m_ChargeAngle = 0.;
 	m_ChargeDist = 0.;
 	m_ChargeAutoPos = true;
+	m_Layout = m_ChargeLayout = NULL;
+	m_DrawCircle = false;
 }
 
 gcpAtom::gcpAtom(OBAtom* atom, double zoom): Atom()
 {
-	m_buf = gtk_text_buffer_new(TextTagTable);
 	m_x = atom->GetX() * zoom;
 	m_y = - atom->GetY() * zoom;
 	m_z = atom->GetZ() * zoom;
@@ -121,7 +113,6 @@ gcpAtom::gcpAtom(OBAtom* atom, double zoom): Atom()
 	gchar* Id = g_strdup_printf("a%d", atom->GetIdx());
 	SetId(Id);
 	g_free(Id);
-//	m_Valence = atom->GetValence();
 	m_HPos = true;
 	m_ascent = 0;
 	m_CHeight = 0.;
@@ -132,13 +123,15 @@ gcpAtom::gcpAtom(OBAtom* atom, double zoom): Atom()
 	m_ChargeAngle = 0.;
 	m_ChargeDist = 0.;
 	m_ChargeAutoPos = true;
+	m_Layout = m_ChargeLayout = NULL;
+	m_DrawCircle = false;
 }
 
 void gcpAtom::SetZ(int Z)
 {
 	Atom::SetZ(Z);
 	m_Element = Element::GetElement (m_Z);
-	if ((m_Valence = m_Element->GetDefaultValence ()))
+	if (m_Valence = m_Element->GetDefaultValence ())
 		m_HPos = GetBestSide();
 	else
 		m_nH = 0;
@@ -204,21 +197,18 @@ bool gcpAtom::GetBestSide()
 	else return Element::BestSide(m_Z);
 }
 
-void gcpAtom::Update ()
+void gcpAtom::Update()
 {
-	if (m_ChargeAuto) {
+	if (m_ChargeAuto)
+	{
 		m_Charge = 0;
 		m_ChargeAuto = false;
 	}
-	if (m_ChargeAutoPos) {
-		NotifyPositionOccupation (m_ChargePos, false);
-		m_ChargePos = 0xff;
-	}
 	int nb, nexplp = 0, nexplu = 0; //nexplp is the number of explicit lone pairs
 	//nexplu is the number of explicit unpaired electrons
 	map<string, Object*>::iterator i;
 	gcpElectron* electron = (gcpElectron*) GetFirstChild (i);
-	while (electron) { 
+	while (electron){ 
 		if (electron->IsPair ())
 			nexplp++;
 		else
@@ -268,6 +258,27 @@ void gcpAtom::Update ()
 	if (pDoc)
 		m_Changed = pDoc->GetView ()->GetNbWidgets ();
 	m_AvailPosCached = false;
+	if (nbonds && GetZ () == 6) {
+		// update large bonds ends
+		gcpBond *bond;
+		gcpBondType type;
+		bool DrawCircle;
+		map<Atom*, Bond*>::iterator i = m_Bonds.begin(), iend = m_Bonds.end ();
+		int nb = 0;
+		while (i != iend)
+		{
+			bond = dynamic_cast<gcpBond*> ((gcpBond*)(*i).second);
+			type = bond->GetType ();
+			if (type == ForeBondType || (type == UpBondType && bond->GetAtom (1) == this))
+				nb++;
+			i++;
+		}
+		DrawCircle = nb > 1;
+		if (DrawCircle != m_DrawCircle) {
+			m_DrawCircle = DrawCircle;
+			m_Changed = true;
+		}
+	}
 }
 	
 bool gcpAtom::IsInCycle(gcpCycle* pCycle)
@@ -278,151 +289,117 @@ bool gcpAtom::IsInCycle(gcpCycle* pCycle)
 	return false;
 }
 
-void gcpAtom::Add(GtkWidget* w)
+void gcpAtom::Add (GtkWidget* w)
 {
-	if (!w) return;
-	if (m_Changed > 0) m_Changed--;
-	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
+	if (!w)
+		return;
+	if (m_Changed > 0)
+		m_Changed--;
+	gcpWidgetData* pData = (gcpWidgetData*) g_object_get_data (G_OBJECT (w), "data");
 	gcpView* pView = pData->View;
-	if (m_ascent <= 0)
-	{
-		PangoContext* pc = pData->View->GetPangoContext();
-		PangoLayout *pl = pango_layout_new(pc);
-		pango_layout_set_text(pl, "l", 1);
-		PangoLayoutIter* iter = pango_layout_get_iter(pl);
-		m_ascent = pango_layout_iter_get_baseline(iter) / PANGO_SCALE;
-		pango_layout_iter_free(iter);
-		g_object_unref(pl);
+	if (m_Layout == NULL) {
+		PangoContext* pc = pView->GetPangoContext();
+		m_Layout = pango_layout_new (pc);
+		pango_layout_set_font_description (m_Layout, pView->GetPangoFontDesc ());
+		pango_layout_set_text (m_Layout, "l", 1);
+		PangoLayoutIter* iter = pango_layout_get_iter (m_Layout);
+		m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
+		pango_layout_iter_free (iter);
 	}
-	if (m_CHeight == 0.)
-	{
-		PangoContext* pc = pData->View->GetPangoContext();
-		PangoLayout *pl = pango_layout_new(pc);
-		pango_layout_set_text(pl, "C", 1);
+	if (m_CHeight == 0.) {
+		pango_layout_set_text (m_Layout, "C", 1);
 		PangoRectangle rect;
-		pango_layout_get_extents(pl, &rect, NULL);
-		m_CHeight =  double(rect.height / PANGO_SCALE) / 2.0;
-		g_object_unref(G_OBJECT(pl));
-	}
-	const gchar* FontName = pView->GetSmallFontName();
-	GtkTextTag* tag = gtk_text_tag_table_lookup(TextTagTable, FontName);
-	if (!tag)
-	{
-		tag = gtk_text_tag_new(FontName);
-		PangoFontDescription* pfd = pView->GetPangoSmallFontDesc();
-		g_object_set(G_OBJECT(tag),
-							"family", pango_font_description_get_family(pfd),
-							"size", pango_font_description_get_size(pfd),
-							NULL);
-		gtk_text_tag_table_add(TextTagTable, tag);
-		g_object_unref((GObject*)tag);
-	}
-	FontName = pView->GetFontName();
-	tag = gtk_text_tag_table_lookup(TextTagTable, FontName);
-	if (!tag)
-	{
-		tag = gtk_text_tag_new(FontName);
-		PangoFontDescription* pfd = pView->GetPangoFontDesc();
-		g_object_set(G_OBJECT(tag),
-							"family", pango_font_description_get_family(pfd),
-							"size", pango_font_description_get_size(pfd),
-							NULL);
-		gtk_text_tag_table_add(TextTagTable, tag);
-		g_object_unref((GObject*)tag);
+		pango_layout_get_extents (m_Layout, &rect, NULL);
+		m_CHeight =  double (rect.height / PANGO_SCALE) / 2.0;
 	}
 	double x, y, xc = 0., yc;
 	m_width =  m_height = 2.0 * pData->Padding;
-	GetCoords(&x, &y);
+	GetCoords (&x, &y);
 	x *= pData->ZoomFactor;
 	y *= pData->ZoomFactor;
-	double dFontHeight = pData->View->GetFontHeight();
 	GnomeCanvasItem *item;
 	GnomeCanvasGroup *group, *chgp;
-	int index = 0;
-	char *hs;
-	PangoContext* pc = pData->View->GetPangoContext();
-	group = GNOME_CANVAS_GROUP(gnome_canvas_item_new(pData->Group, gnome_canvas_group_ext_get_type(), NULL));
-	g_signal_connect(G_OBJECT(group), "event", G_CALLBACK(on_event), w);
-	g_object_set_data(G_OBJECT(group), "object", this);
-	if ((GetZ() != 6) || (GetBondsNumber() == 0))
-	{
-		GtkTextIter start, end;
-		int sw, hw = 0, nw = 0;
-		gtk_text_buffer_get_bounds(m_buf, &start, &end);
-		gtk_text_buffer_delete(m_buf, &start, &end);
-		const gchar* symbol = GetSymbol();
-		gtk_text_buffer_insert_with_tags_by_name(m_buf, &start, symbol, strlen(symbol),pView->GetFontName(), NULL);
-		PangoLayout *pl = pango_layout_new(pc);
-		pango_layout_set_text(pl, symbol, strlen(symbol));
+	group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (pData->Group, gnome_canvas_group_ext_get_type (), NULL));
+	g_signal_connect (G_OBJECT (group), "event", G_CALLBACK (on_event), w);
+	g_object_set_data (G_OBJECT (group), "object", this);
+	if ((GetZ () != 6) || (GetBondsNumber () == 0)) {
+		int sw, sp;
+		const gchar* symbol = GetSymbol (), *text;
+		sw = strlen (symbol);
+		int n = GetAttachedHydrogens ();
+		if (n > 0) {
+			if (n > 1) {
+				gchar const *nb =  g_strdup_printf ("%d", n);
+				int np, nw = strlen (nb);
+				if (m_HPos) {
+					text = g_strconcat (symbol, "H", nb, NULL);
+					np = sw + 1;
+					sp = 0;
+				} else {
+					text = g_strconcat ("H", nb, symbol, NULL);
+					np = 1;
+					sp = np + nw;
+				}
+				pango_layout_set_text (m_Layout, text, -1);
+				PangoAttrList *pal = pango_attr_list_new ();
+				PangoAttribute *attr = pango_attr_font_desc_new (pView->GetPangoSmallFontDesc());
+				attr->start_index = np;
+				attr->end_index = np + nw;
+				pango_attr_list_insert (pal, attr);
+				attr = pango_attr_rise_new (-2 * PANGO_SCALE);
+				attr->start_index = np;
+				attr->end_index = np + nw;
+				pango_attr_list_insert (pal, attr);
+				pango_layout_set_attributes (m_Layout, pal);
+				pango_attr_list_unref (pal);
+			} else {
+				if (m_HPos) {
+					text = g_strconcat (symbol, "H", NULL);
+					sp = 0;
+				} else {
+					text = g_strconcat ("H", symbol, NULL);
+					sp = 1;
+				}
+				pango_layout_set_text (m_Layout, text, -1);
+			}
+		} else {
+			text = g_strdup (symbol);
+			sp = 0;
+			pango_layout_set_text (m_Layout, text, -1);
+		}
 		PangoRectangle rect;
-		pango_layout_get_extents(pl, NULL, &rect);
-		m_width = sw = rect.width / PANGO_SCALE;
-		m_height = rect.height / PANGO_SCALE;
-		m_text_height = dFontHeight;
-		int n = GetAttachedHydrogens();
-		if (n > 0)
-		{
-			pango_layout_set_text(pl, "H", 1);
-			PangoRectangle rect;
-			pango_layout_get_extents(pl, NULL, &rect);
-			hw = rect.width / PANGO_SCALE;
-			index = (m_HPos)? strlen(symbol): 0;
-			gtk_text_buffer_get_iter_at_offset(m_buf, &start, index);
-			gtk_text_buffer_insert_with_tags_by_name(m_buf, &start, "H", 1,pView->GetFontName(), NULL);
-		}
-		g_object_unref(pl);
-		if (n > 1)
-		{
-			pango_context_set_font_description(pc, pData->View->GetPangoSmallFontDesc());
-			pl = pango_layout_new(pc);
-			hs = g_strdup_printf("%d", n);
-			pango_layout_set_text(pl, hs, strlen(hs));
-			pango_layout_get_extents(pl, NULL, &rect);
-			nw = rect.width / PANGO_SCALE;
-			g_object_unref(pl);
-			pango_context_set_font_description(pc, pData->View->GetPangoFontDesc());
-			gtk_text_buffer_get_iter_at_offset(m_buf, &start, index + 1);
-			gtk_text_buffer_insert_with_tags_by_name(m_buf, &start, hs, strlen(hs),pView->GetSmallFontName(), "subscript", NULL);
-			gtk_text_buffer_get_iter_at_offset(m_buf, &start, index + 1);
-			gtk_text_buffer_get_iter_at_offset(m_buf, &end, index + 1 + strlen (hs));
-			gtk_text_buffer_remove_tag_by_name(m_buf, pView->GetFontName(), &start, &end);
-			m_text_height += 2;
-		}
-		m_length = sw + hw + nw;
-		m_lbearing = (m_HPos)? sw / 2: sw / 2 + hw + nw;
+		pango_layout_get_extents (m_Layout, NULL, &rect);
+		m_length = double (rect.width / PANGO_SCALE);
+		m_text_height = m_height = rect.height / PANGO_SCALE;
+		pango_layout_index_to_pos (m_Layout, sp, &rect);
+		int st = rect.x / PANGO_SCALE;
+		pango_layout_index_to_pos (m_Layout, sp + sw, &rect);
+		m_lbearing = (st + rect.x / PANGO_SCALE) / 2.;
 
-		item = gnome_canvas_item_new(
+		item = gnome_canvas_item_new (
 							group,
-							gnome_canvas_rect_ext_get_type(),
+							gnome_canvas_rect_ext_get_type (),
 							"x1", x - m_lbearing - pData->Padding,
 							"y1", y  - m_ascent + m_CHeight - pData->Padding,
 							"x2", x - m_lbearing + m_length + pData->Padding,
 							"y2", y  - m_ascent + m_CHeight + m_height + pData->Padding,
-							"fill_color", (pData->IsSelected(this))? SelectColor: "white",
+							"fill_color", (pData->IsSelected (this))? SelectColor: "white",
 							NULL);
-		g_object_set_data(G_OBJECT(group), "rect", item);
-		g_signal_connect(G_OBJECT(item), "event", G_CALLBACK(on_event), w);
-		g_object_set_data(G_OBJECT(item), "object", this);
+		g_object_set_data (G_OBJECT (group), "rect", item);
+		g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
+		g_object_set_data (G_OBJECT (item), "object", this);
 		
 		item = gnome_canvas_item_new (
 							group,
-							gnome_canvas_rich_text_ext_get_type(),
+							gnome_canvas_pango_get_type (),
 							"x", x - m_lbearing,
 							"y", y - m_ascent + m_CHeight,
-							"width", m_length,
-							"height", m_height,
-							"grow_height", false,
-							"editable", false,
-							"cursor_visible", false,
-							"text_direction", GTK_TEXT_DIR_LTR,
-							NULL);
-		gnome_canvas_rich_text_ext_set_buffer(GNOME_CANVAS_RICH_TEXT_EXT(item), m_buf);
-		g_object_set_data(G_OBJECT(group), "symbol", item);
-		g_object_set_data(G_OBJECT(item), "object", this);
-		g_signal_connect(G_OBJECT(item), "event", G_CALLBACK(on_event), w);
-	}
-	else
-	{
+							"layout", m_Layout,
+						NULL);
+		g_object_set_data (G_OBJECT (group), "symbol", item);
+		g_object_set_data (G_OBJECT (item), "object", this);
+		g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
+	} else {
 		item = gnome_canvas_item_new(
 								group,
 								gnome_canvas_rect_ext_get_type(),
@@ -438,6 +415,21 @@ void gcpAtom::Add(GtkWidget* w)
 		gnome_canvas_item_raise(GNOME_CANVAS_ITEM(group), 1);
 		g_signal_connect(G_OBJECT(item), "event", G_CALLBACK(on_event), w);
 		g_object_set_data(G_OBJECT(item), "object", this);
+		if (m_DrawCircle) {
+			double dx = pData->StereoBondWidth / 2.;
+			item = gnome_canvas_item_new(
+									group,
+									gnome_canvas_ellipse_ext_get_type(),
+									"x1", x - dx,
+									"y1", y - dx,
+									"x2", x + dx,
+									"y2", y + dx,
+									"fill_color",  (pData->IsSelected(this))? SelectColor: Color,
+									NULL);
+			g_object_set_data(G_OBJECT(group), "bullet", item);
+			g_signal_connect(G_OBJECT(item), "event", G_CALLBACK(on_event), w);
+			g_object_set_data(G_OBJECT(item), "object", this);
+		}
 	}
 	pData->Items[this] = group;
 	m_width /= pData->ZoomFactor;
@@ -456,14 +448,15 @@ void gcpAtom::Add(GtkWidget* w)
 		char *fig = NULL;
 		if 	(abs (charge) > 1) {
 			fig = g_strdup_printf ("%d", abs (charge));
-			PangoContext* pc = pData->View->GetPangoContext();
 			PangoRectangle rect;
-			pango_context_set_font_description (pc, pData->View->GetPangoSmallFontDesc ());
-			PangoLayout* pl = pango_layout_new (pc);
-			pango_layout_set_text (pl, fig, -1);
-			pango_layout_get_extents (pl, NULL, &rect);
+			if (!m_ChargeLayout) {
+				PangoContext* pc = pData->View->GetPangoContext();
+				m_ChargeLayout = pango_layout_new (pc);
+				pango_layout_set_font_description (m_ChargeLayout, pData->View->GetPangoSmallFontDesc ());
+			}
+			pango_layout_set_text (m_ChargeLayout, fig, -1);
+			pango_layout_get_extents (m_ChargeLayout, NULL, &rect);
 			m_ChargeWidth = rect.width / PANGO_SCALE;
-			pango_context_set_font_description (pc, pData->View->GetPangoFontDesc ());
 			m_ChargeTWidth = m_ChargeWidth + 1. + ChargeSignSize;
 		} else {
 			m_ChargeWidth = 0.;
@@ -499,10 +492,9 @@ void gcpAtom::Add(GtkWidget* w)
 		if (fig) {
 			item = gnome_canvas_item_new(
 						chgp,
-						gnome_canvas_text_ext_get_type(),
+						gnome_canvas_pango_get_type(),
+						"layout", m_ChargeLayout,
 						"fill_color", (pData->IsSelected(this))? SelectColor: Color,
-						"font-desc", pData->View->GetPangoSmallFontDesc (),
-						"markup", fig,
 						"anchor", GTK_ANCHOR_EAST,
 						"x", x,
 						"y", y,
@@ -580,13 +572,23 @@ void gcpAtom::Update(GtkWidget* w)
 								"x2", x - m_lbearing + m_length + pData->Padding,
 								"y2", y  - m_ascent + m_CHeight + m_text_height + pData->Padding,
 								NULL);
-		} else
+		} else {
 			g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
 									"x1", x - 3,
 									"y1", y - 3,
 									"x2", x + 3,
 									"y2", y + 3,
 								NULL);
+			if (m_DrawCircle) {
+				double dx = pData->StereoBondWidth / 2.;
+				g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "bullet")),
+										"x1", x - dx,
+										"y1", y - dx,
+										"x2", x + dx,
+										"y2", y + dx,
+										NULL);
+			}
+		}
 	}
 	void* item = g_object_get_data (G_OBJECT (group), "charge");
 	int charge = GetCharge ();
@@ -604,23 +606,22 @@ void gcpAtom::Update(GtkWidget* w)
 			char *fig = NULL;
 			if 	(abs (charge) > 1) {
 				fig = g_strdup_printf ("%d", abs (charge));
-				PangoContext* pc = pData->View->GetPangoContext();
 				PangoRectangle rect;
-				pango_context_set_font_description (pc, pData->View->GetPangoSmallFontDesc ());
-				PangoLayout* pl = pango_layout_new (pc);
-				pango_layout_set_text (pl, fig, -1);
-				pango_layout_get_extents (pl, NULL, &rect);
+				if (!m_ChargeLayout) {
+					PangoContext* pc = pData->View->GetPangoContext();
+					m_ChargeLayout = pango_layout_new (pc);
+					pango_layout_set_font_description (m_ChargeLayout, pData->View->GetPangoSmallFontDesc ());
+				}
+				pango_layout_set_text (m_ChargeLayout, fig, -1);
+				pango_layout_get_extents (m_ChargeLayout, NULL, &rect);
 				m_ChargeWidth = rect.width / PANGO_SCALE;
-				pango_context_set_font_description (pc, pData->View->GetPangoFontDesc ());
 			} else
 				m_ChargeWidth = 0.;
 			m_ChargeTWidth = m_ChargeWidth + 1. + ChargeSignSize;
 			if (figure == NULL && fig != NULL) {
 				figure = gnome_canvas_item_new(
 							GNOME_CANVAS_GROUP (item),
-							gnome_canvas_text_ext_get_type(),
-							"fill_color", (pData->IsSelected(this))? SelectColor: Color,
-							"font-desc", pData->View->GetPangoSmallFontDesc (),
+							gnome_canvas_pango_get_type(),
 							"anchor", GTK_ANCHOR_EAST,
 							NULL);
 				g_object_set_data (G_OBJECT (group), "figure", figure);
@@ -652,7 +653,7 @@ void gcpAtom::Update(GtkWidget* w)
 			yc = y - ChargeSignHeight / 2. + ChargeYAlign - ChargeSignSize + 1.;
 			if (fig) {
 				g_object_set (G_OBJECT (figure),
-							"markup", fig,
+							"layout", m_ChargeLayout,
 							"x", x,
 							"y", y,
 							NULL);
@@ -701,14 +702,15 @@ void gcpAtom::Update(GtkWidget* w)
 			char *fig = NULL;
 			if 	(abs (charge) > 1) {
 				fig = g_strdup_printf ("%d", abs (charge));
-				PangoContext* pc = pData->View->GetPangoContext();
+				if (!m_ChargeLayout) {
+					PangoContext* pc = pData->View->GetPangoContext();
+					m_ChargeLayout = pango_layout_new (pc);
+					pango_layout_set_font_description (m_ChargeLayout, pData->View->GetPangoSmallFontDesc ());
+				}
+				pango_layout_set_text (m_ChargeLayout, fig, -1);
 				PangoRectangle rect;
-				pango_context_set_font_description (pc, pData->View->GetPangoSmallFontDesc ());
-				PangoLayout* pl = pango_layout_new (pc);
-				pango_layout_set_text (pl, fig, -1);
-				pango_layout_get_extents (pl, NULL, &rect);
+				pango_layout_get_extents (m_ChargeLayout, NULL, &rect);
 				m_ChargeWidth = rect.width / PANGO_SCALE;
-				pango_context_set_font_description (pc, pData->View->GetPangoFontDesc ());
 				m_ChargeTWidth = m_ChargeWidth + 2 + ChargeSignSize;
 			} else {
 				m_ChargeWidth = 0.;
@@ -744,10 +746,9 @@ void gcpAtom::Update(GtkWidget* w)
 			if (fig) {
 				item = gnome_canvas_item_new(
 							chgp,
-							gnome_canvas_text_ext_get_type(),
+							gnome_canvas_pango_get_type(),
+							"layout", m_ChargeLayout,
 							"fill_color", (pData->IsSelected(this))? SelectColor: Color,
-							"font-desc", pData->View->GetPangoSmallFontDesc (),
-							"markup", fig,
 							"anchor", GTK_ANCHOR_EAST,
 							"x", x,
 							"y", y,
@@ -798,9 +799,6 @@ void gcpAtom::Update(GtkWidget* w)
 		if (item)
 			gtk_object_destroy (GTK_OBJECT (item));
 			g_object_set_data ((GObject*)group, "charge", NULL);
-			g_object_set_data ((GObject*)group, "figure", NULL);
-			g_object_set_data ((GObject*)group, "circle", NULL);
-			g_object_set_data ((GObject*)group, "sign", NULL);
 	}
 	map<string, Object*>::iterator i;
 	Object* electron = GetFirstChild (i);
@@ -842,30 +840,14 @@ void gcpAtom::UpdateAvailablePositions ()
 		while ((n != m_AngleList.end()) && (*n < angle)) n++;
 		m_AngleList.insert(n, angle);
 		i++;
-		if ((m_AvailPos & CHARGE_SW) && (angle >= 180.0 - ATOM_EPSILON) &&
-			(angle <= 270.0 + ATOM_EPSILON))
-			m_AvailPos -= CHARGE_SW;
-		if ((m_AvailPos & CHARGE_SE) && (((angle >= 270.0 - ATOM_EPSILON) &&
-			(angle <= 360.0 + ATOM_EPSILON)) || (fabs(angle) < ATOM_EPSILON)))
-			m_AvailPos -= CHARGE_SE;
-		if ((m_AvailPos & CHARGE_S) && (angle >= 225.0 - ATOM_EPSILON) &&
-			(angle <= 315.0 + ATOM_EPSILON))
-			m_AvailPos -= CHARGE_S;
-		if ((m_AvailPos & CHARGE_NW) && (angle >= 90.0 - ATOM_EPSILON) &&
-			(angle <= 180.0 + ATOM_EPSILON))
-			m_AvailPos -= CHARGE_NW;
-		if ((m_AvailPos & CHARGE_NE) && (((angle >= 0.0 - ATOM_EPSILON) &&
-			(angle <= 90.0 + ATOM_EPSILON)) || (fabs(angle - 360.0) < ATOM_EPSILON)))
-			m_AvailPos -= CHARGE_NE;
-		if ((m_AvailPos & CHARGE_N) && (angle >= 45.0 - ATOM_EPSILON) &&
-			(angle <= 135.0 + ATOM_EPSILON))
-			m_AvailPos -= CHARGE_N;
-		if ((m_AvailPos & CHARGE_W) && ((angle <= 225.0 + ATOM_EPSILON) &&
-			(angle >= 135.0 - ATOM_EPSILON)))
-			m_AvailPos -= CHARGE_W;
-		if ((m_AvailPos & CHARGE_E) && ((angle >= 315.0 - ATOM_EPSILON) ||
-			(angle <= 45.0 + ATOM_EPSILON)))
-			m_AvailPos -= CHARGE_E;
+		if ((m_AvailPos & CHARGE_SW) && (angle >= 180.0) && (angle <= 270.0)) m_AvailPos -= CHARGE_SW;
+		if ((m_AvailPos & CHARGE_SE) && (((angle >= 270.0) && (angle <= 360.0)) || (fabs(angle) < 0.1))) m_AvailPos -= CHARGE_SE;
+		if ((m_AvailPos & CHARGE_S) && (angle >= 225.0) && (angle <= 315.0)) m_AvailPos -= CHARGE_S;
+		if ((m_AvailPos & CHARGE_NW) && (angle >= 90.0) && (angle <= 180.0)) m_AvailPos -= CHARGE_NW;
+		if ((m_AvailPos & CHARGE_NE) && (((angle >= 0.0) && (angle <= 90.0)) || (fabs(angle - 360.0) < 0.1))) m_AvailPos -= CHARGE_NE;
+		if ((m_AvailPos & CHARGE_N) && (angle >= 45.0) && (angle <= 135.0)) m_AvailPos -= CHARGE_N;
+		if ((m_AvailPos & CHARGE_W) && ((angle <= 225.0) && (angle >= 135.0))) m_AvailPos -= CHARGE_W;
+		if ((m_AvailPos & CHARGE_E) && ((angle >= 315.0) || (angle <= 45.0))) m_AvailPos -= CHARGE_E;
 	}
 	m_AngleList.push_back ((angle = m_AngleList.front ()) + 360.0);
 	m_InterBonds.clear ();
@@ -1110,21 +1092,34 @@ void gcpAtom::SetSelected(GtkWidget* w, int state)
 	gpointer item;
 	gchar *color, *chargecolor;
 	
-	switch (state)
-	{	
-		case SelStateUnselected: color = "white"; chargecolor = "black"; break;
-		case SelStateSelected: chargecolor = color = SelectColor; break;
-		case SelStateUpdating: chargecolor = color = AddColor; break;
-		case SelStateErasing: chargecolor = color = DeleteColor; break;
-		default: color = "white"; chargecolor = "black"; break;
+	switch (state) {	
+	case SelStateUnselected:
+		color = "white";
+		chargecolor = "black";
+		break;
+	case SelStateSelected:
+		chargecolor = color = SelectColor;
+		break;
+	case SelStateUpdating:
+		chargecolor = color = AddColor;
+		break;
+	case SelStateErasing:
+		chargecolor = color = DeleteColor;
+		break;
+	default:
+		color = "white";
+		chargecolor = "black";
+		break;
 	}
 	g_object_set (G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
 				"fill_color", color, NULL);
-	if ((item = g_object_get_data(G_OBJECT(group), "figure")))
+	if (item = g_object_get_data(G_OBJECT(group), "bullet"))
 		g_object_set (item, "fill_color", chargecolor, NULL);
-	if ((item = g_object_get_data(G_OBJECT(group), "circle")))
+	if (item = g_object_get_data(G_OBJECT(group), "figure"))
+		g_object_set (item, "fill_color", chargecolor, NULL);
+	if (item = g_object_get_data(G_OBJECT(group), "circle"))
 		g_object_set (item, "outline_color", chargecolor, NULL);
-	if ((item = g_object_get_data(G_OBJECT(group), "sign")))
+	if (item = g_object_get_data(G_OBJECT(group), "sign"))
 		g_object_set (item, "outline_color", chargecolor, NULL);
 	Object::SetSelected (w, state);
 }
@@ -1157,61 +1152,65 @@ void gcpAtom::BuildItems (gcpWidgetData* pData)
 {
 	GnomeCanvasGroup* group = pData->Items[this];
 	void* item;
-	GtkTextIter start, end;
-	double dFontHeight = pData->View->GetFontHeight();
-	PangoContext* pc = pData->View->GetPangoContext();
+	gcpView* pView = pData->View;
 	double x, y;
 	m_width =  m_height = 2.0 * pData->Padding;
-	GetCoords(&x, &y);
+	GetCoords (&x, &y);
 	x *= pData->ZoomFactor;
 	y *= pData->ZoomFactor;
-	if ((GetZ() != 6) || (GetBondsNumber() == 0))
-	{
-		int sw, hw = 0, nw = 0;
-		int index = 0;
-		char *hs;
-		gtk_text_buffer_get_bounds(m_buf, &start, &end);
-		gtk_text_buffer_delete(m_buf, &start, &end);
-		const gchar* symbol = GetSymbol();
-		gtk_text_buffer_insert_with_tags_by_name(m_buf, &start, symbol, strlen(symbol),pData->View->GetFontName(), NULL);
-		PangoLayout *pl = pango_layout_new(pc);
-		pango_layout_set_text(pl, symbol, strlen(symbol));
+	if ((GetZ() != 6) || (GetBondsNumber() == 0)) {
+		int sw, sp;
+		const gchar* symbol = GetSymbol (), *text;
+		sw = strlen (symbol);
+		int n = GetAttachedHydrogens ();
+		PangoAttrList *pal = pango_attr_list_new ();
+		if (n > 0) {
+			if (n > 1) {
+				gchar const *nb =  g_strdup_printf ("%d", n);
+				int np, nw = strlen (nb);
+				if (m_HPos) {
+					text = g_strconcat (symbol, "H", nb, NULL);
+					np = sw + 1;
+					sp = 0;
+				} else {
+					text = g_strconcat ("H", nb, symbol, NULL);
+					np = 1;
+					sp = np + nw;
+				}
+				pango_layout_set_text (m_Layout, text, -1);
+				PangoAttribute *attr = pango_attr_font_desc_new (pView->GetPangoSmallFontDesc());
+				attr->start_index = np;
+				attr->end_index = np + nw;
+				pango_attr_list_insert (pal, attr);
+				attr = pango_attr_rise_new (-2 * PANGO_SCALE);
+				attr->start_index = np;
+				attr->end_index = np + nw;
+				pango_attr_list_insert (pal, attr);
+			} else {
+				if (m_HPos) {
+					text = g_strconcat (symbol, "H", NULL);
+					sp = 0;
+				} else {
+					text = g_strconcat ("H", symbol, NULL);
+					sp = 1;
+				}
+				pango_layout_set_text (m_Layout, text, -1);
+			}
+			pango_layout_set_attributes (m_Layout, pal);
+			pango_attr_list_unref (pal);
+		} else {
+			text = g_strdup (symbol);
+			sp = 0;
+			pango_layout_set_text (m_Layout, text, -1);
+		}
 		PangoRectangle rect;
-		pango_layout_get_extents(pl, NULL, &rect);
-		m_width = sw = rect.width / PANGO_SCALE;
-		m_height = rect.height / PANGO_SCALE;
-		m_text_height = dFontHeight;
-		int n = GetAttachedHydrogens();
-		if (n > 0)
-		{
-			pango_layout_set_text(pl, "H", 1);
-			PangoRectangle rect;
-			pango_layout_get_extents(pl, NULL, &rect);
-			hw = rect.width / PANGO_SCALE;
-			index = (m_HPos)? strlen(symbol): 0;
-			gtk_text_buffer_get_iter_at_offset(m_buf, &start, index);
-			gtk_text_buffer_insert_with_tags_by_name(m_buf, &start, "H", 1,pData->View->GetFontName(), NULL);
-		}
-		g_object_unref(pl);
-		if (n > 1)
-		{
-			pango_context_set_font_description(pc, pData->View->GetPangoSmallFontDesc());
-			pl = pango_layout_new(pc);
-			hs = g_strdup_printf("%d", n);
-			pango_layout_set_text(pl, hs, strlen(hs));
-			pango_layout_get_extents(pl, NULL, &rect);
-			nw = rect.width / PANGO_SCALE;
-			g_object_unref(pl);
-			pango_context_set_font_description(pc, pData->View->GetPangoFontDesc());
-			gtk_text_buffer_get_iter_at_offset(m_buf, &start, index + 1);
-			gtk_text_buffer_insert_with_tags_by_name(m_buf, &start, hs, strlen(hs),pData->View->GetSmallFontName(), "subscript", NULL);
-			gtk_text_buffer_get_iter_at_offset(m_buf, &start, index + 1);
-			gtk_text_buffer_get_iter_at_offset(m_buf, &end, index + 1 + strlen (hs));
-			gtk_text_buffer_remove_tag_by_name(m_buf, pData->View->GetFontName(), &start, &end);
-			m_text_height += 2;
-		}
-		m_length = sw + hw + nw;
-		m_lbearing = (m_HPos)? sw / 2: sw / 2 + hw + nw;
+		pango_layout_get_extents (m_Layout, NULL, &rect);
+		m_length = double (rect.width / PANGO_SCALE);
+		m_text_height = m_height = rect.height / PANGO_SCALE;
+		pango_layout_index_to_pos (m_Layout, sp, &rect);
+		int st = rect.x / PANGO_SCALE;
+		pango_layout_index_to_pos (m_Layout, sp + sw, &rect);
+		m_lbearing = (st + rect.x / PANGO_SCALE) / 2.;
 
 		item = g_object_get_data (G_OBJECT (group), "rect");
 		g_object_set (item,
@@ -1226,31 +1225,26 @@ void gcpAtom::BuildItems (gcpWidgetData* pData)
 			g_object_set (item,
 							"x", x - m_lbearing,
 							"y", y - m_ascent + m_CHeight,
-							"width", m_length,
-							"height", m_height,
 							NULL);
 		else {
 			item = gnome_canvas_item_new (
 							group,
-							gnome_canvas_rich_text_ext_get_type(),
+							gnome_canvas_pango_get_type(),
+							"layout", m_Layout,
 							"x", x - m_lbearing,
 							"y", y - m_ascent + m_CHeight,
-							"width", m_length,
-							"height", m_height,
-							"grow_height", false,
-							"editable", false,
-							"cursor_visible", false,
-							"text_direction", GTK_TEXT_DIR_LTR,
 							NULL);
-			gnome_canvas_rich_text_ext_set_buffer(GNOME_CANVAS_RICH_TEXT_EXT(item), m_buf);
-			g_object_set_data(G_OBJECT(group), "symbol", item);
-			g_object_set_data(G_OBJECT(item), "object", this);
-			g_signal_connect(G_OBJECT(item), "event", G_CALLBACK(on_event), pData->Canvas);
-			gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(group));
+			g_object_set_data (G_OBJECT (group), "symbol", item);
+			g_object_set_data (G_OBJECT (item), "object", this);
+			g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), pData->Canvas);
+			gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (group));
 		}
-	}
-	else
-	{
+		item = g_object_get_data (G_OBJECT (group), "bullet");
+		if (item) {
+			gtk_object_destroy (GTK_OBJECT (item));
+			g_object_set_data (G_OBJECT (group), "bullet", NULL);
+		}
+	} else {
 		item = g_object_get_data (G_OBJECT (group), "rect");
 		g_object_set (item,
 							"x1", x - 3,
@@ -1259,16 +1253,38 @@ void gcpAtom::BuildItems (gcpWidgetData* pData)
 							"y2", y + 3,
 							NULL);
 		item = g_object_get_data (G_OBJECT (group), "symbol");
-		if (item){
+		if (item) {
 			gtk_object_destroy (GTK_OBJECT (item));
 			g_object_set_data (G_OBJECT (group), "symbol", NULL);
 		}
+		item = g_object_get_data (G_OBJECT (group), "bullet");
+		if (m_DrawCircle) {
+			if (!item) {
+				double dx = pData->StereoBondWidth / 2.;
+				item = gnome_canvas_item_new(
+										group,
+										gnome_canvas_ellipse_ext_get_type (),
+										"x1", x - dx,
+										"y1", y - dx,
+										"x2", x + dx,
+										"y2", y + dx,
+										"fill_color",  (pData->IsSelected (this))? SelectColor: Color,
+										NULL);
+				g_object_set_data (G_OBJECT (group), "bullet", item);
+				g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), pData->Canvas);
+				g_object_set_data (G_OBJECT (item), "object", this);
+			}
+		} else if (item) {
+			gtk_object_destroy (GTK_OBJECT (item));
+			g_object_set_data (G_OBJECT (group), "bullet", NULL);
+		}
 		m_length = m_text_height = 0;
 		gnome_canvas_item_lower_to_bottom (GNOME_CANVAS_ITEM (group));
 	}
 	m_width /= pData->ZoomFactor;
 	m_height /= pData->ZoomFactor;
-	if (m_Changed > 0) m_Changed--;
+	if (m_Changed > 0)
+		m_Changed--;
 }
 	
 double gcpAtom::GetYAlign ()
@@ -1546,31 +1562,3 @@ void gcpAtom::SetCharge (int charge)
 	m_ChargeAuto = false;
 	Update ();
 }
-
-void gcpAtom::Transform2D (Matrix2D& m, double x, double y)
-{
-	Atom::Transform2D (m, x, y);
-	// Now transform electrons
-	map<string, Object*>::iterator i;
-	Object* electron = GetFirstChild (i);
-	while (electron) {
-		electron->Transform2D (m, x, y);
-		electron = GetNextChild (i);
-	}
-	if (GetCharge ()) {
-		m_ChargeAutoPos = false; // should be fixed before 0.6.4
-/*		if (m_ChargeAutoPos) {
-			if (m_ChargePos > 0)
-				NotifyPositionOccupation (m_ChargePos, false);
-			m_ChargePos = 0xff;
-			Update ();
-		} else {*/
-			double xc = cos (m_ChargeAngle), yc = - sin (m_ChargeAngle);
-			m.Transform (xc, yc);
-			m_ChargeAngle = atan2 (- yc, xc);
-			if (m_ChargeAngle < 0)
-				m_ChargeAngle += 2 * M_PI;
-			SetChargePosition (0, FALSE, m_ChargeAngle, m_ChargeDist);
-//		}
-	}
-}
