@@ -4,9 +4,9 @@
  * GChemPaint library
  * atom.h 
  *
- * Copyright (C) 2001-2004
+ * Copyright (C) 2001-2005
  *
- * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -79,14 +79,14 @@ public :
 	virtual void SetZ(int Z);
 	void AddBond(Bond* pBond);
 	void RemoveBond(Bond* pBond);
-	void Update();
+	virtual void Update();
 	virtual void Add(GtkWidget* w);
 	virtual void Update(GtkWidget* w);
 	int GetTotalBondsNumber(); //take bond order into account
 	int GetAttachedHydrogens() {return m_nH;}
 	bool GetBestSide();
 	bool IsInCycle(gcpCycle* pCycle);
-	virtual int GetChargePosition(char& Pos, double Angle, double& x, double& y);
+	virtual int GetChargePosition(unsigned char& Pos, double Angle, double& x, double& y);
 	virtual int GetAvailablePosition(double& x, double& y);
 	virtual bool GetPosition(double angle, double& x, double& y);
 	virtual xmlNodePtr Save (xmlDocPtr xml);
@@ -105,9 +105,10 @@ public :
 	bool HasImplicitElectronPairs ();
 	bool MayHaveImplicitUnpairedElectrons ();
 	void AddElectron (gcpElectron* electron);
-	void NotifyPositionOccupation (char pos, bool occupied);
-	void SetChargePosition (char Pos, bool def, double angle = 0., double distance = 0.);
+	void NotifyPositionOccupation (unsigned char pos, bool occupied);
+	void SetChargePosition (unsigned char Pos, bool def, double angle = 0., double distance = 0.);
 	char GetChargePosition (double *Angle, double *Dist);
+	void SetCharge (int charge);
 
 private:
 	void BuildItems (gcpWidgetData* pData);
@@ -125,20 +126,22 @@ private:
 	bool m_HPos; //false = left, true = right
 	bool m_ChargeAuto;
 	int m_Changed; //update needs regenerate the buffer
-	GtkTextBuffer *m_buf;
 	int m_ascent;
-	int m_lbearing;
+	double m_lbearing;
 	double m_CHeight;
 	unsigned char m_AvailPos; //available standard positions for charge and electrons representations
 	unsigned char m_OccupiedPos;
 	bool m_AvailPosCached;
-	char m_ChargePos;
+	unsigned char m_ChargePos;
 	bool m_ChargeAutoPos;
 	double m_ChargeAngle;
 	double m_ChargeDist;
+	double m_ChargeWidth, m_ChargeTWidth, m_ChargeXOffset, m_ChargeYOffset;
 	list<double> m_AngleList;
 	map<double, double> m_InterBonds; /* positions betwen bonds. First  value is the
 	angle between the two bonds and second value is the direction */
+	PangoLayout *m_Layout, *m_ChargeLayout;
+	bool m_DrawCircle;
 };
 
 #endif // GCHEMPAINT_ATOM_H
