@@ -4,7 +4,7 @@
  * GChemPaint library
  * bond.cc 
  *
- * Copyright (C) 2001-2004
+ * Copyright (C) 2001-2005
  *
  * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
@@ -227,11 +227,21 @@ Object* gcpBond::GetAtomAt(double x, double y, double z)
 
 bool gcpBond::SaveNode(xmlDocPtr xml, xmlNodePtr node)
 {
-	switch(m_type)
-	{
-		case UpBondType: xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"up"); break;
-		case DownBondType: xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"down"); break;
-		case UndeterminedBondType: xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"undetermined"); break;
+	switch(m_type) {
+	case UpBondType:
+		xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"up");
+		break;
+	case DownBondType:
+		xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"down");
+		break;
+	case ForeBondType:
+		xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"fore");
+		break;
+	case UndeterminedBondType:
+		xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"undetermined");
+		break;
+	default:
+		break;
 	}
 	return true;
 }
@@ -246,6 +256,8 @@ bool gcpBond::LoadNode (xmlNodePtr node)
 		SetType (UpBondType);
 	else if (!strcmp (tmp, "down"))
 		SetType (DownBondType);
+	else if (!strcmp (tmp, "fore"))
+		SetType (ForeBondType);
 	else if (!strcmp (tmp, "undetermined"))
 		SetType (UndeterminedBondType);
 	else
@@ -346,12 +358,22 @@ void gcpBond::SetSelected(GtkWidget* w, int state)
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
 	GnomeCanvasGroup* group = pData->Items[this];
 	gchar* color;
-	switch (state)
-	{	
-		case SelStateUnselected: color = Color; break;
-		case SelStateSelected: color = SelectColor; break;
-		case SelStateUpdating: color = AddColor; break;
-		case SelStateErasing: color = DeleteColor; break;
+	switch (state) {	
+	case SelStateUnselected:
+		color = Color;
+		break;
+	case SelStateSelected:
+		color = SelectColor;
+		break;
+	case SelStateUpdating:
+		color = AddColor;
+		break;
+	case SelStateErasing:
+		color = DeleteColor;
+		break;
+	default:
+		color = Color;
+		break;
 	}
 	void *obj = g_object_get_data(G_OBJECT(group), "path");
 	switch (GetType())
@@ -360,6 +382,7 @@ void gcpBond::SetSelected(GtkWidget* w, int state)
 		case UndeterminedBondType:
 			g_object_set(obj, "outline_color", color, NULL);
 			break;
+		case ForeBondType:
 		case UpBondType:
 		case DownBondType:
 			g_object_set(obj, "fill_color", color, NULL);
@@ -376,9 +399,8 @@ void gcpBond::Add(GtkWidget* w)
 	if (!(pAtom1 = GetAtom(1))) return;
 	unsigned char order = GetOrder();
 	if (order == 0) return;
-	double x1, y1, x2, y2;
 	GnomeCanvasGroup* group;
-	GnomeCanvasItem* item;
+	GnomeCanvasItem* item = NULL;
 	pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
 	group = GNOME_CANVAS_GROUP(gnome_canvas_item_new(pData->Group, gnome_canvas_group_ext_get_type(), NULL));
 	g_signal_connect(G_OBJECT(group), "event", G_CALLBACK(on_event), w);
@@ -396,6 +418,7 @@ void gcpBond::Add(GtkWidget* w)
 									"width_units", pData->BondWidth,
 									NULL);
 			break;
+		case ForeBondType:
 		case UpBondType:
 		case DownBondType:
 			item = gnome_canvas_item_new(
@@ -459,6 +482,7 @@ bool gcpBond::ReplaceAtom(gcpAtom* oldAtom, gcpAtom* newAtom)
 		if (m_Begin && m_End)
 			m_Begin->AddBond(this);
 	}
+	return true;
 }
 
 GnomeCanvasPathDef* gcpBond::BuildPathDef (gcpWidgetData* pData)
@@ -527,6 +551,17 @@ GnomeCanvasPathDef* gcpBond::BuildPathDef (gcpWidgetData* pData)
 			}
 			break;
 		}
+		case ForeBondType:
+			GetLine2DCoords(1, &x1, &y1, &x2, &y2);
+			x = sqrt(square(x2 - x1) + square(y2 - y1));
+			dx = (y1 - y2) / x * pData->StereoBondWidth / 2;
+			dy = (x2 - x1) / x * pData->StereoBondWidth / 2;
+			gnome_canvas_path_def_moveto (path, x1 * pData->ZoomFactor + dx, y1 * pData->ZoomFactor + dy);
+			gnome_canvas_path_def_lineto (path, x2 * pData->ZoomFactor + dx, y2 * pData->ZoomFactor + dy);
+			gnome_canvas_path_def_lineto (path, x2 * pData->ZoomFactor -dx, y2 * pData->ZoomFactor -dy);
+			gnome_canvas_path_def_lineto (path, x1 * pData->ZoomFactor -dx, y1 * pData->ZoomFactor -dy);
+			gnome_canvas_path_def_closepath (path);
+			break;
 		case UndeterminedBondType:
 		{
 			GetLine2DCoords(1, &x1, &y1, &x2, &y2);
