@@ -4,7 +4,7 @@
  * GChemPaint library
  * docprop.h 
  *
- * Copyright (C) 2002-2004
+ * Copyright (C) 2002-2005
  *
  * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
@@ -27,11 +27,13 @@
 #ifndef GCHEMPAINT_DOC_PROP_H
 #define GCHEMPAINT_DOC_PROP_H
 
-#include "dialog.h"
+#include <gcu/dialog.h>
+
+using namespace gcu;
 
 class gcpDocument;
 
-class gcpDocPropDlg: public gcpDialog
+class gcpDocPropDlg: public Dialog
 {
 public:
 	gcpDocPropDlg(gcpDocument* pDoc);
