@@ -4,7 +4,9 @@
  * GChemPaint library
  * electron.cc
  *
- * Copyright (C) 2004-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2005
+ *
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -30,7 +32,7 @@
 #include "widgetdata.h"
 #include "libgcpcanvas/gcp-canvas-rect-ellipse.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
-#include <cmath>
+#include <math.h>
 
 #define POSITION_NE 1
 #define POSITION_NW 2
@@ -219,13 +221,22 @@ void gcpElectron::SetSelected(GtkWidget* w, int state)
 	GnomeCanvasGroup* group = pData->Items[this];
 	gchar *color;
 	
-	switch (state)
-	{	
-		case SelStateUnselected: color = const_cast<char*> ("black"); break;
-		case SelStateSelected: color = SelectColor; break;
-		case SelStateUpdating: color = AddColor; break;
-		case SelStateErasing: color = DeleteColor; break;
-		default: color = const_cast<char*> ("black"); break;
+	switch (state) {	
+	case SelStateUnselected:
+		color = "black";
+		break;
+	case SelStateSelected:
+		color = SelectColor;
+		break;
+	case SelStateUpdating:
+		color = AddColor;
+		break;
+	case SelStateErasing:
+		color = DeleteColor;
+		break;
+	default:
+		color = "black";
+		break;
 	}
 	g_object_set (G_OBJECT(g_object_get_data(G_OBJECT(group), "0")),
 				"fill_color", color, NULL);
@@ -241,31 +252,31 @@ xmlNodePtr gcpElectron::Save (xmlDocPtr xml)
 	if (m_Pos) {
 		switch (m_Pos) {
 		case POSITION_NE:
-			buf = const_cast<char*> ("ne");
+			buf = "ne";
 			break;
 		case POSITION_NW:
-			buf = const_cast<char*> ("nw");
+			buf = "nw";
 			break;
 		case POSITION_N:
-			buf = const_cast<char*> ("n");
+			buf = "n";
 			break;
 		case POSITION_SE:
-			buf = const_cast<char*> ("se");
+			buf = "se";
 			break;
 		case POSITION_SW:
-			buf = const_cast<char*> ("sw");
+			buf = "sw";
 			break;
 		case POSITION_S:
-			buf = const_cast<char*> ("s");
+			buf = "s";
 			break;
 		case POSITION_E:
-			buf = const_cast<char*> ("e");
+			buf = "e";
 			break;
 		case POSITION_W:
-			buf = const_cast<char*> ("w");
+			buf = "w";
 			break;
 		default:
-			buf = const_cast<char*> ("def"); // should not occur
+			buf = "def"; // should not occur
 		}
 		xmlNewProp (node, (xmlChar*) "position", (xmlChar*) buf);
 	} else {
@@ -344,14 +355,3 @@ bool gcpElectron::OnSignal (SignalId Signal, Object *Child)
 	pDoc->FinishOperation ();
 	return false;
 }
-
-void gcpElectron::Transform2D (Matrix2D& m, double x, double y)
-{
-	double a = m_Angle * M_PI / 180.;
-	double xc = cos (a), yc = - sin (a);
-	m.Transform (xc, yc);
-	a = atan2 (- yc, xc) * 180. / M_PI;
-	if (a < 0)
-		a += 360;
-	SetPosition (0, a, m_Dist);
-}
