@@ -4,7 +4,9 @@
  * GChemPaint library
  * fragment.cc 
  *
- * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2005
+ *
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -34,9 +36,8 @@
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include "libgcpcanvas/gcp-canvas-rect-ellipse.h"
 #include "libgcpcanvas/gcp-canvas-bpath.h"
-#include "libgcpcanvas/gcp-canvas-text.h"
 #include <gcu/element.h>
-#include <cmath>
+#include <math.h>
 
 extern GtkTextTagTable *TextTagTable;
 
@@ -50,7 +51,7 @@ gcpFragment::gcpFragment(): gcpTextObject(FragmentType)
 	m_EndAtom = gtk_text_buffer_create_mark(m_buf, "ae0", &start, true);
 	m_lbearing = 0;
 	m_CHeight = 0.;
-	SetId(const_cast<char*> ("f1"));
+	SetId("f1");
 }
 
 gcpFragment::gcpFragment(double x, double y): gcpTextObject(x, y, FragmentType)
@@ -63,7 +64,7 @@ gcpFragment::gcpFragment(double x, double y): gcpTextObject(x, y, FragmentType)
 	m_EndAtom = gtk_text_buffer_create_mark(m_buf, "ae0", &start, true);
 	m_lbearing = 0;
 	m_CHeight = 0.;
-	SetId(const_cast<char*> ("f1"));
+	SetId("f1");
 }
 
 gcpFragment::~gcpFragment()
@@ -449,20 +450,31 @@ void gcpFragment::SetSelected(GtkWidget* w, int state)
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
 	GnomeCanvasGroup* group = pData->Items[this];
 	gchar *chargecolor, *color;
-	switch (state)
-	{	
-		case SelStateUnselected: color = const_cast<char*> ("white"); chargecolor = const_cast<char*> ("black"); break;
-		case SelStateSelected: chargecolor = color = SelectColor; break;
-		case SelStateUpdating: chargecolor = color = AddColor; break;
-		case SelStateErasing: chargecolor = color = DeleteColor; break;
-		default: color = const_cast<char*> ("white"); chargecolor = const_cast<char*> ("black"); break;
+	switch (state) {	
+	case SelStateUnselected:
+		color = "white";
+		chargecolor = "black";
+		break;
+	case SelStateSelected:
+		chargecolor = color = SelectColor;
+		break;
+	case SelStateUpdating:
+		chargecolor = color = AddColor;
+		break;
+	case SelStateErasing:
+		chargecolor = color = DeleteColor;
+		break;
+	default:
+		color = "white";
+		chargecolor = "black";
+		break;
 	}
 	g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
 				"fill_color", color, NULL);
 	gpointer item;
-	if ((item = g_object_get_data(G_OBJECT(group), "circle")))
+	if (item = g_object_get_data(G_OBJECT(group), "circle"))
 		g_object_set (item, "outline_color", chargecolor, NULL);
-	if ((item = g_object_get_data(G_OBJECT(group), "sign")))
+	if (item = g_object_get_data(G_OBJECT(group), "sign"))
 		g_object_set (item, "outline_color", chargecolor, NULL);
 }
 
@@ -1094,7 +1106,7 @@ int gcpFragment::GetElementAtIter(GtkTextIter* start, GtkTextIter* end)
 	gtk_text_buffer_get_iter_at_offset(m_buf, end, offset + 3);
 	while (gtk_text_iter_compare(start, end))
 	{
-		if ((Z = Element::Z(gtk_text_buffer_get_text(m_buf, start, end, false)))) return Z;
+		if (Z = Element::Z(gtk_text_buffer_get_text(m_buf, start, end, false))) return Z;
 		gtk_text_iter_backward_char(end);
 	}
 	return 0;
@@ -1142,7 +1154,7 @@ int gcpFragment::GetChargePosition(gcpFragmentAtom* pAtom, unsigned char& Pos, d
 		if ((result & CHARGE_SW) && (((angle >= 0.0) && (angle <= 90.0)) || (fabs(angle - 360.0) < 0.1))) result -= CHARGE_SW;
 		if ((result & CHARGE_S) && (angle >= 45.0) && (angle <= 135.0)) result -= CHARGE_S;
 		if ((result & CHARGE_E) && ((angle <= 225.0) && (angle >= 135.0))) result -= CHARGE_E;
-		if ((result & CHARGE_W) && ((angle >= 315.0) || (angle <= 45.0))) result -= CHARGE_W;
+		if ((result & CHARGE_W) && (angle >= 315.0) || (angle <= 45.0)) result -= CHARGE_W;
 	}
 	if (Pos == 0xff) {
 		if (result) {
