@@ -4,7 +4,9 @@
  * GChemPaint library
  * mendeleiev.cc 
  *
- * Copyright (C) 2001-2004 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2005
+ *
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -33,7 +35,7 @@ static void on_element(GtkWidget *widget, guint Z, gcpMendeleievDlg *pBox)
 	pBox->OnElement(Z);
 }
 
-gcpMendeleievDlg::gcpMendeleievDlg(gcpApplication* App, int Z): gcpDialog(App, PKGDATADIR"/ui/mendeleiev.glade", "mendeleiev")
+gcpMendeleievDlg::gcpMendeleievDlg(gcpApplication* App, int Z): Dialog(App, DATADIR"/gchempaint-unstable/ui/mendeleiev.glade", "mendeleiev")
 {
 	GtkWidget* frame = glade_xml_get_widget(xml, "frame");
 	periodic = (GtkPeriodic*)gtk_periodic_new();
@@ -50,15 +52,15 @@ gcpMendeleievDlg::gcpMendeleievDlg(gcpApplication* App, int Z): gcpDialog(App, P
 
 gcpMendeleievDlg::~gcpMendeleievDlg()
 {
-	m_App->ToggleMenu("Mendeleiev", false);
-	m_App->SetDialog("Mendeleiev", NULL);
+	reinterpret_cast<gcpApplication*> (m_App)->ToggleMenu("Mendeleiev", false);
+	reinterpret_cast<gcpApplication*> (m_App)->SetDialog("Mendeleiev", NULL);
 }
 
 void gcpMendeleievDlg::OnElement(int Z)
 {
-	m_App->SetCurZ(Z);
+	reinterpret_cast<gcpApplication*> (m_App)->SetCurZ(Z);
 	if (apply) apply(m_data, Z);
-	SetElement (m_App, Z);
+	SetElement (reinterpret_cast<gcpApplication*> (m_App), Z);
 }
 
 void gcpMendeleievDlg::SetCallBack(void (*f)(void*, int), void* data)
