@@ -4,7 +4,7 @@
  * GChemPaint library
  * operation.h 
  *
- * Copyright (C) 2002-2004
+ * Copyright (C) 2002-2005
  *
  * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
@@ -50,7 +50,7 @@ void gcpOperation::Add(unsigned type)
 
 void gcpOperation::Delete (unsigned type)
 {
-	xmlNodePtr node = m_Nodes[type]->children, child;
+	xmlNodePtr node = m_Nodes[type]->children;
 	char* Id;
 	while (node)
 	{
