@@ -4,7 +4,9 @@
  * GChemPaint library
  * reaction-operator.cc 
  *
- * Copyright (C) 2004-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004
+ *
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -29,8 +31,8 @@
 #include "widgetdata.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include "libgcpcanvas/gcp-canvas-rect-ellipse.h"
-#include "libgcpcanvas/gcp-canvas-text.h"
-#include <cmath>
+#include "libgcpcanvas/gcp-canvas-pango.h"
+#include <math.h>
 
 gcpReactionOperator::gcpReactionOperator (): Object (ReactionOperatorType)
 {
@@ -60,10 +62,10 @@ void gcpReactionOperator::Add (GtkWidget* w)
 	g_signal_connect (G_OBJECT (group), "event", G_CALLBACK (on_event), w);
 	g_object_set_data (G_OBJECT (group), "object", this);
 	const gchar* symbol = "+";
-	PangoLayout *pl = pango_layout_new (pc);
-	pango_layout_set_text (pl, symbol, strlen (symbol));
+	m_Layout = pango_layout_new (pc);
+	pango_layout_set_text (m_Layout, symbol, strlen (symbol));
 	PangoRectangle rect;
-	pango_layout_get_extents (pl, &rect, NULL);
+	pango_layout_get_extents (m_Layout, &rect, NULL);
 	width = rect.width / PANGO_SCALE;
 	height =  rect.height / PANGO_SCALE;
 	item = gnome_canvas_item_new (
@@ -80,11 +82,10 @@ void gcpReactionOperator::Add (GtkWidget* w)
 	g_object_set_data (G_OBJECT (item), "object", this);
 	item = gnome_canvas_item_new (
 						group,
-						gnome_canvas_text_ext_get_type (),
-						"text", "+",
+						gnome_canvas_pango_get_type (),
+						"layout", m_Layout,
 						"x", rint (x),
 						"y", rint (y),
-						"font", szFontName,
 						"anchor", GTK_ANCHOR_CENTER,
 						"fill_color", (pData->IsSelected (this))? SelectColor: Color,
 						NULL);
