@@ -4,7 +4,9 @@
  * GChemPaint library
  * reaction.cc 
  *
- * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2005
+ *
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -28,11 +30,11 @@
 #include "reaction-step.h"
 #include "document.h"
 #include "view.h"
-#include <cmath>
+#include <math.h>
 
 gcpReaction::gcpReaction (): Object (ReactionType)
 {
-	SetId (const_cast<char*> ("rxn1"));
+	SetId ("rxn1");
 }
 
 gcpReaction::~gcpReaction ()
@@ -44,7 +46,7 @@ gcpReaction::~gcpReaction ()
 	gcpReactionArrow *arrow;
 	gcpDocument *pDoc = reinterpret_cast<gcpDocument *> (GetDocument ());
 	gcpOperation *pOp = pDoc->GetCurrentOperation ();
-	while ((pObj = GetFirstChild (i))) {
+	while (pObj = GetFirstChild (i)) {
 		if (pObj->GetType () == ReactionArrowType) {
 			arrow = reinterpret_cast<gcpReactionArrow*> (pObj);
 			arrow->SetStartStep (NULL);
@@ -80,7 +82,7 @@ bool gcpReaction::Build (list<Object*>& Children) throw (invalid_argument)
 	list<Object*> Others;
 	map<double, Object*> Left, Right;
 	double x0, y0, x1, y1, x, y, xpos, ypos, l;
-	bool horiz = true;
+	bool horiz = true;;
 	ArtDRect *rect, srect;
 	for (i = Children.begin (); i != end; i++) {
 		// It might be better to use the objects coordinates there
@@ -391,9 +393,11 @@ double gcpReaction::GetYAlign ()
 	Object *pObj;
 	pObj = GetFirstChild (i);
 	double y = DBL_MAX, new_y;
-	while (pObj)
+	while (pObj) {
 		if (pObj->GetType () == ReactionStepType)
 			if ((new_y = pObj->GetYAlign ()) < y)
-				y = new_y;			
+				y = new_y;	
+		pObj = GetNextChild (i);
+	}			
 	return y;
 }
