@@ -4,7 +4,9 @@
  * GChemPaint library
  * view.cc
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2005
+ *
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -32,12 +34,12 @@
 #include "text.h"
 #include "libgcpcanvas/gprintable.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
+#include <math.h>
 #include <unistd.h>
 #include <pango/pango-context.h>
 #include <gdk/gdkkeysyms.h>
 #include <libgnomeprint/gnome-print-job.h>
-#include <cmath>
-#include <clocale>
+#include <locale.h>
 #include <fstream>
 
 /*
@@ -149,9 +151,9 @@ static bool on_destroy(GtkWidget *widget, gcpView * pView)
 	return true;
 }
 
-static bool on_size(GtkWidget *widget, GtkAllocation *alloc, gcpView * pView)
+static bool on_size (GtkWidget *widget, GtkAllocation *alloc, gcpView * pView)
 {
-	pView->OnSize(alloc->width, alloc->height);
+	pView->OnSize (widget, alloc->width, alloc->height);
 	return true;
 }
 
@@ -312,7 +314,7 @@ bool gcpView::OnEvent(GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 		pActiveTool->OnDrag(x, y, event->button.state);
 		return true;
 	case GDK_BUTTON_RELEASE:
-		switch (event->button.button)
+	switch (event->button.button)
 		{
 		case 1:
 			if (!m_Dragging) break;
@@ -320,15 +322,12 @@ bool gcpView::OnEvent(GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 			pActiveTool->OnRelease(x, y, event->button.state);
 			m_pDoc->GetApplication()->ClearStatus();
 			return true;
-		default:
-			break;
 		}
 		break;
 	default:
 		break;
 	}
 	return false;
-	
 }
 
 void gcpView::AddObject(Object* pObject)
@@ -354,7 +353,6 @@ GtkWidget* gcpView::CreateNewWidget()
 		m_pData->View = this;
 		gnome_canvas_set_pixels_per_unit(GNOME_CANVAS(m_pWidget), 1);
 		gnome_canvas_set_scroll_region(GNOME_CANVAS(m_pWidget), 0, 0, m_width, m_height);
-//		gtk_widget_set_size_request(m_pWidget, m_width, m_height);//FIXME: probably necessary when embedded
 		m_pData->ZoomFactor = DefaultZoomFactor;
 		m_pData->Padding = m_DefaultPadding * DefaultZoomFactor;
 		m_pData->BondWidth = m_DefaultBondWidth * DefaultZoomFactor;
@@ -370,6 +368,7 @@ GtkWidget* gcpView::CreateNewWidget()
 		m_pData->StereoBondWidth = m_DefaultStereoBondWidth * DefaultZoomFactor;
 		m_pData->ObjectPadding = m_DefaultObjectPadding * DefaultZoomFactor;
 		m_pData->SignPadding = m_DefaultSignPadding * DefaultZoomFactor;
+		m_pData->Zoom = 1.0;
 		m_pData->Background = gnome_canvas_item_new(
 									gnome_canvas_root(GNOME_CANVAS(m_pWidget)),
 									gnome_canvas_rect_get_type(),
@@ -386,6 +385,7 @@ GtkWidget* gcpView::CreateNewWidget()
 		g_signal_connect(G_OBJECT(m_pData->Background), "event", G_CALLBACK(on_event), m_pWidget);
 		g_signal_connect(G_OBJECT(m_pWidget), "destroy", G_CALLBACK(on_destroy), this);
 		g_signal_connect(G_OBJECT(m_pWidget), "size_allocate", G_CALLBACK(on_size), this);
+		g_signal_connect(G_OBJECT(m_pWidget), "realize", G_CALLBACK(gnome_canvas_gcp_update_bounds), this);
 		gtk_widget_show(m_pWidget);
 		m_Widgets.push_back(m_pWidget);
 		if (pWidget)
@@ -598,10 +598,12 @@ void gcpView::OnReceive(GtkClipboard* clipboard, GtkSelectionData* selection_dat
 	guint *DataType = (clipboard == gtk_clipboard_get(GDK_SELECTION_CLIPBOARD))? &ClipboardDataType: &ClipboardDataType1;
 	g_return_if_fail((selection_data->target == gdk_atom_intern (targets[*DataType].target, FALSE)));
 	if (pActiveTool->OnReceive(clipboard, selection_data, *DataType)) return;
-	else if (pActiveTool)
+//	if (ToolsDlg) ToolsDlg->Select(SelectId);
+	else if (pActiveTool/* && pActiveTool->Activate(false)*/)
 	{
 		pApp->ActivateTool("Select", true);
 		pActiveTool = pApp->GetActiveTool();
+//		if (pActiveTool) pActiveTool->Activate(true);
 	}
 	if (!pActiveTool || (pActiveTool != pApp->GetTool("Select"))) return;
 	xmlDocPtr xml;
@@ -770,18 +772,12 @@ bool gcpView::OnKeyRelease(GtkWidget* w, GdkEventKey* event)
 	return false;
 }
 	
-bool gcpView::OnSize(int width, int height)
+bool gcpView::OnSize (GtkWidget *w, int width, int height)
 {
-	if ((width == m_width) && (height == m_height)) return true;
-	std::list<GtkWidget*>::iterator i;
-	m_height = height;
-	m_width = width;
-	for (i = m_Widgets.begin(); i != m_Widgets.end(); i++)
-	{
-		gnome_canvas_set_scroll_region(GNOME_CANVAS(*i), 0, 0, width, height);
-		gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(*i), "data");
-		if (pData->Background) g_object_set(G_OBJECT(pData->Background), "x2", (double) width, "y2", (double) height, NULL);
-	}
+	gcpWidgetData* pData = (gcpWidgetData*) g_object_get_data (G_OBJECT (w), "data");
+	gnome_canvas_set_scroll_region (GNOME_CANVAS (w), 0, 0, (double) width / pData->Zoom, (double) height / pData->Zoom);
+	if (pData->Background)
+		g_object_set (G_OBJECT (pData->Background), "x2", (double) width / pData->Zoom, "y2", (double) height / pData->Zoom, NULL);
 	return true;
 }
 
@@ -790,9 +786,11 @@ void gcpView::UpdateSize(double x1, double y1, double x2, double y2)
 	if (x1 < 0.0) x2 -= x1;
 	if (y1 < 0.0) y2 -= y1;
 	list<GtkWidget*>::iterator i;
-	if ((x2 > m_width) || (y2 > m_height))
-		for (i = m_Widgets.begin(); i != m_Widgets.end(); i++)
-			gtk_widget_set_size_request(*i,(int)ceil(x2), (int)ceil(y2));
+	if ((x2 != m_width) || (y2 != m_height))
+		for (i = m_Widgets.begin(); i != m_Widgets.end(); i++) {
+			gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(*i), "data");
+			gtk_widget_set_size_request(*i,(int)ceil(x2 * pData->Zoom), (int)ceil(y2 * pData->Zoom));
+		}
 	if ((x1 < 0.0) || (y1 < 0.0))
 	{
 		x1 = - x1;
@@ -1083,5 +1081,13 @@ void gcpView::EnsureSize ()
 	while (canvas->idle_id)
 		gtk_main_iteration();
 	gnome_canvas_update_now (canvas);
-	g_signal_emit_by_name (m_pWidget, "update_bounds");
+	if (GTK_WIDGET_REALIZED (m_pWidget))
+		g_signal_emit_by_name (m_pWidget, "update_bounds");
+}
+
+void gcpView::Zoom (double zoom)
+{
+	m_pData->Zoom = zoom;
+	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), zoom);
+	EnsureSize ();
 }
