@@ -6,7 +6,7 @@
  *
  * Copyright (C) 2001-2004
  *
- * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -45,8 +45,8 @@ class gcpView
 {
 public:
 	//Constructor and destructor
-	gcpView(gcpDocument *pDoc, bool Embedded);
-	virtual ~gcpView();
+	gcpView (gcpDocument *pDoc, bool Embedded);
+	virtual ~gcpView ();
 
 	//Interface	
 public:
@@ -78,7 +78,7 @@ public:
 	void OnCutSelection(GtkWidget* w, GtkClipboard* clipboard);
 	bool OnKeyPress(GtkWidget* w, GdkEventKey* event);
 	bool OnKeyRelease(GtkWidget* w, GdkEventKey* event);
-	bool OnSize(int width, int height);
+	bool OnSize (GtkWidget *w, int width, int height);
 	void UpdateSize(double x1, double y1, double x2, double y2);
 	void SetGnomeCanvasRichTextActive(GnomeCanvasRichTextExt* item);
 	bool PrepareUnselect();
@@ -88,8 +88,10 @@ public:
 	int GetNbWidgets () {return m_Widgets.size ();}
 	void ExportImage (const char* filename, const char* type);
 	double GetArrowPadding () {return m_ArrowPadding;}
-	double GetStoechiometryPadding () {return m_StoechiometryPadding;}
+	double GetStoichiometryPadding () {return m_StoichiometryPadding;}
 	double GetBaseLineOffset () {return m_BaseLineOffset;}
+	void EnsureSize ();
+	void Zoom (double zoom);
 
 	//Implementation
 private:
@@ -116,7 +118,7 @@ private:
 	double m_DefaultObjectPadding;
 	double m_DefaultSignPadding;
 	double m_ArrowPadding;
-	double m_StoechiometryPadding;
+	double m_StoichiometryPadding;
 	double m_BaseLineOffset;
 	int m_width, m_height;
 	double m_lastx, m_lasty;
