@@ -1,7 +1,9 @@
 /* 
  * pango-layout-to-svg.c 
  *
- * Copyright (C) 2004 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2005
+ *
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -21,9 +23,6 @@
 
 #include <pango/pango-layout.h>
 #include <libxml/tree.h>
-#define GTK_TEXT_USE_INTERNAL_UNSUPPORTED_API
-#include <gtk/gtktextlayout.h>
-#include <libart_lgpl/art_render_svp.h>
 #include <math.h>
 
 void pango_layout_to_svg (PangoLayout* layout, xmlDocPtr doc, xmlNodePtr node, double x, double y)
@@ -37,6 +36,7 @@ void pango_layout_to_svg (PangoLayout* layout, xmlDocPtr doc, xmlNodePtr node, d
 	const char *text = pango_layout_get_text (layout);
 	GString *str;
 	PangoLayoutIter *iter = pango_layout_get_iter (layout);
+	/*PangoAlignment align = pango_layout_get_alignment (layout);*/
 
 	child = xmlNewDocNode (doc, NULL, (const xmlChar*) "text", NULL);
 	xmlAddChild (node, child);
