@@ -4,7 +4,9 @@
  * GChemPaint atoms plugin
  * plugin.cc 
  *
- * Copyright (C) 2004 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004
+ *
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -42,7 +44,7 @@ gcpAtomsPlugin::~gcpAtomsPlugin()
 
 void gcpAtomsPlugin::Populate (gcpApplication* App)
 {
-	App->AddUI (PKGDATADIR"/ui/atoms.xml");
+	App->AddUI (DATADIR"/gchempaint-unstable/ui/atoms.xml");
 	new gcpElementTool (App);
 	new gcpChargeTool (App, "ChargePlus");
 	new gcpChargeTool (App, "ChargeMinus");
