@@ -70,6 +70,19 @@ protected:
 	virtual void SetType(gcpBond* pBond);
 };
 
+class gcpForeBondTool: public gcpBondTool
+{
+public:
+	gcpForeBondTool(gcpApplication *App);
+	virtual ~gcpForeBondTool();
+
+protected:
+	virtual void Draw();
+	virtual void UpdateBond();
+	virtual void FinalizeBond();
+	virtual void SetType(gcpBond* pBond);
+};
+
 class gcpDownBondTool: public gcpBondTool
 {
 public:
