@@ -28,6 +28,7 @@
 #include "plugin.h"
 #include "lib/application.h"
 #include "bondtool.h"
+#include "delocalizedtool.h"
 
 gcpBondsPlugin plugin;
 
@@ -41,9 +42,11 @@ gcpBondsPlugin::~gcpBondsPlugin()
 
 void gcpBondsPlugin::Populate (gcpApplication* App)
 {
-	App->AddUI (DATADIR"/gchempaint/ui/bonds.xml");
+	App->AddUI (DATADIR"/gchempaint-unstable/ui/bonds.xml");
 	new gcpBondTool(App);
 	new gcpUpBondTool (App);
 	new gcpDownBondTool (App);
+	new gcpForeBondTool (App);
 	new gcpSquiggleBondTool (App);
+	new gcpDelocalizedTool (App);
 }
