@@ -4,7 +4,9 @@
  * GChemPaint templates plugin
  * plugin.cc 
  *
- * Copyright (C) 2004 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004
+ *
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -39,13 +41,13 @@ gcpTemplatesPlugin::gcpTemplatesPlugin(): gcpPlugin()
 	xmlNodePtr node;
 	const char *name;
 	char *path;
-	GDir* dir = g_dir_open (PKGDATADIR"/templates", 0, NULL);
+	GDir* dir = g_dir_open (DATADIR"/gchempaint/templates", 0, NULL);
 	xmlIndentTreeOutput = true;
 	xmlKeepBlanksDefault (0);
 	if (dir) {
-		while ((name = g_dir_read_name (dir))) {
+		while (name = g_dir_read_name (dir)) {
 			if (strcmp (name + strlen (name) - 4, ".xml")) continue;
-			name = g_strconcat (PKGDATADIR"/templates/", name, NULL);
+			name = g_strconcat (DATADIR"/gchempaint/templates/", name, NULL);
 			doc = xmlParseFile (name);
 			docs.insert (doc);
 			node = doc->children;
@@ -58,7 +60,7 @@ gcpTemplatesPlugin::gcpTemplatesPlugin(): gcpPlugin()
 	dir = g_dir_open (path, 0, NULL);
 	if (dir) {
 		bool is_user_templates;
-		while ((name = g_dir_read_name (dir))) {
+		while (name = g_dir_read_name (dir)) {
 			if (strcmp (name + strlen (name) - 4, ".xml")) continue;
 			is_user_templates = !strcmp (name, "templates.xml");
 			name = g_strconcat (path, "/", name, NULL);
@@ -99,7 +101,7 @@ gcpTemplatesPlugin::~gcpTemplatesPlugin()
 
 void gcpTemplatesPlugin::Populate (gcpApplication* App)
 {
-	App->AddUI (GLADEDIR"/templates.xml");
+	App->AddUI (DATADIR"/gchempaint-unstable/ui/templates.xml");
 	new gcpTemplateTool (App);
 	new gcpTemplateTree (App);
 }
