@@ -4,7 +4,9 @@
  * GChemPaint
  * main.cc 
  *
- * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2005
+ *
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -32,6 +34,10 @@ extern "C" {
 	void gnome_authentication_manager_init ();
 }
 
+#warning "the following lines should be removed for stable releases"
+#undef PACKAGE
+#define PACKAGE "gchempaint-unstable" 
+
 gboolean cb_print_version (const gchar *option_name, const gchar *value, gpointer data, GError **error)
 {
 	char *version = g_strconcat (_("GChemPaint version: "), VERSION, NULL);
@@ -43,7 +49,7 @@ gboolean cb_print_version (const gchar *option_name, const gchar *value, gpointe
 
 static GOptionEntry entries[] = 
 {
-  { "version", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, (void*) cb_print_version, "prints GChemPaint version", NULL },
+  { "version", 'v', 0, G_OPTION_ARG_CALLBACK, (void*) cb_print_version, "prints GChemPaint version", NULL },
    { NULL }
 };
 
