@@ -94,7 +94,7 @@ void on_print(GtkWidget* widget, gcpStandaloneApp* App)
 
 void on_show_elements(GtkWidget* widget, gcpStandaloneApp* App)
 {
-	gcpDialog *dlg = App->GetDialog ("Mendeleiev");
+	Dialog *dlg = App->GetDialog ("Mendeleiev");
 	if (dlg)
 		dlg->Destroy ();
 	else
@@ -146,6 +146,58 @@ static bool on_key_press(GtkWidget* widget, GdkEventKey* ev, gcpStandaloneApp* A
 	return App->OnKeyPressed(widget, ev);
 }
 
+/*static void on_prefs(GtkWidget* widget, gcpStandaloneApp* App)
+{
+	GtkFontSelectionDialog* dlg = (GtkFontSelectionDialog*)gtk_font_selection_dialog_new("");
+	gtk_dialog_run(GTK_DIALOG(dlg));
+	gtk_widget_destroy(GTK_WIDGET(dlg));
+}*/
+
+static void on_zoom_400 (GtkWidget* widget, gcpStandaloneApp* App)
+{
+	App->Zoom (4.);
+}
+
+static void on_zoom_300 (GtkWidget* widget, gcpStandaloneApp* App)
+{
+	App->Zoom (3.);
+}
+
+static void on_zoom_200 (GtkWidget* widget, gcpStandaloneApp* App)
+{
+	App->Zoom (2.);
+}
+
+static void on_zoom_150 (GtkWidget* widget, gcpStandaloneApp* App)
+{
+	App->Zoom (1.5);
+}
+
+static void on_zoom_100 (GtkWidget* widget, gcpStandaloneApp* App)
+{
+	App->Zoom (1.);
+}
+
+static void on_zoom_75 (GtkWidget* widget, gcpStandaloneApp* App)
+{
+	App->Zoom (.75);
+}
+
+static void on_zoom_50 (GtkWidget* widget, gcpStandaloneApp* App)
+{
+	App->Zoom (.5);
+}
+
+static void on_zoom_25 (GtkWidget* widget, gcpStandaloneApp* App)
+{
+	App->Zoom (.3);
+}
+
+static void on_zoom (GtkWidget* widget, gcpStandaloneApp* App)
+{
+	App->Zoom (0.);
+}
+
 static void on_close_all(GtkWidget* widget, gcpStandaloneApp* App)
 {
 	App->OnCloseAll ();
@@ -261,13 +313,32 @@ static GtkActionEntry entries[] = {
 	  { "Copy", GTK_STOCK_COPY, N_("_Copy"), "<control>C",
 		  N_("Copy the selection"), G_CALLBACK (on_copy_selection) },
 	  { "Paste", GTK_STOCK_PASTE, N_("_Paste"), "<control>V",
-		  N_("Paste the clipborad"), G_CALLBACK (on_paste_selection) },
+		  N_("Paste the clipboard"), G_CALLBACK (on_paste_selection) },
 	  { "Delete", GTK_STOCK_CLEAR, N_("C_lear"), NULL,
 		  N_("Clear the selection"), G_CALLBACK (on_delete_selection) },
 	  { "SelectAll", NULL, N_("Select _All"), "<control>A",
 		  N_("Select everything"), G_CALLBACK (on_select_all) },
   { "ViewMenu", NULL, N_("_View") },
 	  { "ToolbarMenu", NULL, N_("_Toolbars") },
+	  { "ZoomMenu", NULL, N_("_Zoom") },
+		  { "400%", NULL, N_("_400%"), "<control>4",
+			  N_("Zoom to 400%"), G_CALLBACK (on_zoom_400) },
+		  { "300%", NULL, N_("_300%"), "<control>3",
+			  N_("Zoom to 300%"), G_CALLBACK (on_zoom_300) },
+		  { "200%", NULL, N_("_200%"), "<control>2",
+			  N_("Zoom to 200%"), G_CALLBACK (on_zoom_200) },
+		  { "150%", NULL, N_("150%"), "<control>6",
+			  N_("Zoom to 150%"), G_CALLBACK (on_zoom_150) },
+		  { "100%", NULL, N_("_100%"), "<control>1",
+			  N_("Zoom to 100%"), G_CALLBACK (on_zoom_100) },
+ 		  { "75%", NULL, N_("_75%"), "<control>7",
+			  N_("Zoom to 75%"), G_CALLBACK (on_zoom_75) },
+		  { "50%", NULL, N_("_50%"), "<control>5",
+			  N_("Zoom to 50%"), G_CALLBACK (on_zoom_50) },
+		  { "25%", NULL, N_("25%"),  "<control>8",
+			  N_("Zoom to 25%"), G_CALLBACK (on_zoom_25) },
+		  { "Zoom", NULL, N_("_Zoom to...%"), "<control>M",
+			  N_("Open Zoom Dialog Box"), G_CALLBACK (on_zoom) },
   { "WindowsMenu", NULL, N_("_Windows") },
 	  { "SaveAll", GTK_STOCK_SAVE, N_("_Save All"), "<shift><control>L",
 		  N_("Save all open files"), G_CALLBACK (on_save_all) },
@@ -324,6 +395,18 @@ static const char *ui_description =
 "      <menu action='ToolbarMenu'>"
 "      </menu>"
 "      <menuitem action='Mendeleiev'/>"
+"      <separator name='view-sep1'/>"
+"      <menu action='ZoomMenu'>"
+"        <menuitem action='400%'/>"
+"        <menuitem action='300%'/>"
+"        <menuitem action='200%'/>"
+"        <menuitem action='150%'/>"
+"        <menuitem action='100%'/>"
+"        <menuitem action='75%'/>"
+"        <menuitem action='50%'/>"
+"        <menuitem action='25%'/>"
+"        <menuitem action='Zoom'/>"
+"      </menu>"
 "    </menu>"
 "    <menu action='WindowsMenu'>"
 "      <menuitem action='SaveAll'/>"
@@ -374,6 +457,10 @@ static const char *ui_web_description =
 "  </menubar>"
 "</ui>";
 
+#warning "the following lines should be removed for stable releases"
+#undef PACKAGE
+#define PACKAGE "gchempaint-unstable" 
+
 gcpStandaloneApp::gcpStandaloneApp(): gcpApplication()
 {
 	GtkWidget *vbox;
@@ -763,7 +850,7 @@ void gcpStandaloneApp::AddToolbar (BonoboUINode* Node, map<unsigned, GtkWidget*>
 						char* filename = NULL;
 						if (bonobo_ui_node_has_attr (Child, "pixname")) {
 							name = bonobo_ui_node_get_attr (Child, "pixname");
-							filename = g_strconcat (DATADIR"/gchempaint/ui/", name, NULL);
+							filename = g_strconcat (DATADIR"/gchempaint-unstable/ui/", name, NULL);
 							bonobo_ui_node_free_string (name);
 						}
 						if (filename) {
